
from ..runenvtool import RunEnvTool

class futoinTool( RunEnvTool ):
    def autoDetect( self, config ) :
        return self._autoDetectByCfg( config, 'futoin.json' )

    def initEnv( self, env ):
        self._have_tool = True

    def updateProjectConfig( self, config, updates ) :
        def updater( json ):
            for f in ( 'name', 'version' ):
                if f in updates :
                        json[f] = updates[f]

        return self._updateJSONConfig( 'futoin.json', updater )




import os

from ..buildtool import BuildTool

class gemTool( BuildTool ):
    def getDeps( self ) :
        return [ 'ruby' ]
        
    def uninstallTool( self, env ):
        pass
    
    def initEnv( self, env ) :
        if env['rubyVer'] == self.SYSTEM_VER:
            gemDir = os.path.join(os.environ['HOME'], '.gem')
            gemDir = env.setdefault('gemDir', gemDir)
            os.environ['GEM_HOME'] = gemDir
            
            self._addEnvPath( 'GEM_PATH', gemDir)
            self._addBinPath( os.path.join(gemDir, 'bin'))

        super(gemTool, self).initEnv( env )




from ..buildtool import BuildTool
from .gemtoolmixin import GemToolMixIn

class bundlerTool( GemToolMixIn, BuildTool ):
    def autoDetect( self, config ) :
        return self._autoDetectByCfg(
                config,
                [ 'Gemfile' ]
        )

    def onPrepare( self, config ):
        self._callExternal( [ config['env']['bundlerBin'], 'install' ] )
        
    def initEnv( self, env ) :
        super(bundlerTool, self).initEnv( env, 'bundle' )




from __future__ import print_function, absolute_import

import os, json, stat, shutil
from collections import OrderedDict

class UtilMixIn( object ):
    def _updateEnvFromOutput( self, env_to_set ):
        env_to_set = env_to_set.split( "\n" )

        for e in env_to_set:
            if not e: break
            n, v = e.split('=', 1)
            os.environ[n] = v

    def _autoDetectByCfg( self, config, file_name ) :
        if self._name in config.get( 'tools', [] ) :
            return True
        
        if type( file_name ) is type( '' ):
            file_name = [ file_name ]

        for f in file_name :
            if os.path.exists( f ) :
                return True
        
        return False

    def _updateJSONConfig( self, file_name, updater, indent=2, separators=(',', ': ') ) :
        with open(file_name, 'r') as content_file:
            content = content_file.read()
            object_pairs_hook = lambda pairs: OrderedDict( pairs )
            content = json.loads( content, object_pairs_hook=object_pairs_hook )
            
        updater( content )
        
        with open(file_name, 'w') as content_file:
            content =  json.dumps( content, indent=indent, separators=separators )
            content_file.write( content )
            content_file.write( "\n" )
            
        return [ file_name ]
    
    def _updateTextFile( self, file_name, updater ) :
        with open(file_name, 'r') as content_file:
            content = content_file.read()
            
        content = updater( content )
        
        with open(file_name, 'w') as content_file:
            content_file.write( content )
            
        return [ file_name ]
    
    def _isExternalToolsSetup( self, env ):
        return env['externalSetup']['installTools']
    
    def _rmTree( self, dir ):
        os.chmod( dir, stat.S_IRWXU )
        for ( path, dirs, files ) in os.walk( dir ) :
            for id in dirs + files :
                os.chmod( os.path.join( path, id ), stat.S_IRWXU )
        shutil.rmtree(dir)        




import json
from collections import OrderedDict

from ..buildtool import BuildTool
from .npmtoolmixin import NpmToolMixIn

class bowerTool( NpmToolMixIn, BuildTool ):
    def autoDetect( self, config ) :
        return self._autoDetectByCfg( config, 'bower.json' )

    def loadConfig( self, config ) :
        with open('bower.json', 'r') as content_file:
            content = content_file.read()
            object_pairs_hook = lambda pairs: OrderedDict( pairs )
            content = json.loads( content, object_pairs_hook=object_pairs_hook )
        f = 'name'
        if f in content and f not in config:
            config[f] = content[f]

    def updateProjectConfig( self, config, updates ) :
        def updater( json ):
            f = 'name'
            if f in updates :
                    json[f] = updates[f]
                    
            # version is deprecated
            if 'version' in json:
                del json['version']

        return self._updateJSONConfig( 'bower.json', updater )
    
    def onPrepare( self, config ):
        bowerBin = config['env']['bowerBin']
        self._callExternal( [ bowerBin, 'install' ] )

    def onPackage( self, config ):
        bowerBin = config['env']['bowerBin']
        # TODO: not sure
        self._callExternal( [ bowerBin, 'install', '--production' ] )
    




import os

from ..runenvtool import RunEnvTool

class sshTool( RunEnvTool ):
    def _installTool( self, env ):
        self._requireDeb(['openssh-client'])
        self._requireRpm(['openssh'])
    
    



from ..runtimetool import RuntimeTool
from .bashtoolmixin import BashToolMixIn

class rubyTool( BashToolMixIn, RuntimeTool ):
    def getDeps( self ) :
        return [ 'rvm' ]

    def _installTool( self, env ):
        ruby_ver = env['rubyVer']
        
        if ruby_ver == self.SYSTEM_VER:
            self._systemDeps()
            return
        
        self._buildDeps()
        self._callExternal([
            env['rvmBin'], 'install', ruby_ver, '--autolibs=read-only'
        ])
        self._callExternal([
            env['rvmBin'], 'cleanup', 'all'
        ])

    def updateTool( self, env ):
        if env['rubyVer'] != self.SYSTEM_VER:
            self._installTool( env )
        
    def uninstallTool( self, env ):
        ruby_ver = env['rubyVer']
        
        if ruby_ver != self.SYSTEM_VER:
            self._callExternal([
                env['rvmBin'], 'uninstall', env['rubyVer']
            ])
            self._have_tool = False

    def _envNames( self ) :
        return ['rubyVer', 'rubyBin' ]

    def initEnv( self, env ) :
        ruby_ver = env.setdefault('rubyVer', self.SYSTEM_VER)

        rvm_dir = env['rvmDir']

        try:
            env_to_set = self._callBash( env,
                'source {0}/scripts/rvm && \
                rvm use {1} >/dev/null && \
                env | grep "rvm"'.format(rvm_dir, ruby_ver),
                verbose = False
            )
        except:
            return

        if env_to_set :
            self._updateEnvFromOutput(env_to_set)
            super(rubyTool, self).initEnv( env )
            
    def _buildDeps( self ):
        # APT
        #---
        self._requireDeb([
            'build-essential',
            'gawk',
            'libssl-dev',
            'make',
            'libc6-dev',
            'zlib1g-dev',
            'libyaml-dev',
            'libsqlite3-dev',
            'sqlite3',
            'autoconf',
            'libgmp-dev',
            'libgdbm-dev',
            'libncurses5-dev',
            'automake',
            'libtool',
            'bison',
            'pkg-config',
            'libffi-dev',
            'libgmp-dev',
            'libreadline-dev',
        ])
        

        # Extra repo before the rest
        #---
        self._requireYum(['epel-release'])

        self._requireRpm([
            'binutils',
            'patch',
            'libyaml-devel',
            'autoconf',
            'gcc',
            'gcc-c++',
            'glibc-devel',
            'readline-devel',
            'zlib-devel',
            'libffi-devel',
            'openssl-devel',
            'automake',
            'libtool',
            'bison',
            'sqlite-devel',
            'make',
            'm4',
            'gdbm-devel',
            'libopenssl-devel',
            'sqlite3-devel',
        ])
        
    def _systemDeps( self ):
        self._requirePackages(['ruby'])




from ..buildtool import BuildTool
from ..runenvtool import RunEnvTool

class dockerTool( BuildTool, RunEnvTool ):
    def autoDetect( self, config ) :
        return self._autoDetectByCfg(
                config,
                [ 'Dockerfile' ]
        )

    def getOrder( self ):
        return 60

    def onBuild( self, config ):
        self._callExternal( [ config['env']['dockerBin'], 'build' ] )
        




import json
from collections import OrderedDict

from ..buildtool import BuildTool

class npmTool( BuildTool ):
    def autoDetect( self, config ) :
        return self._autoDetectByCfg( config, 'package.json' )
    
    def getDeps( self ) :
        return [ 'node' ]
        
    def updateTool( self, env ):
        self._callExternal( [ env['npmBin'], 'update', '-g', 'npm' ] )
        
    def uninstallTool( self, env ):
        pass
    
    def loadConfig( self, config ) :
        with open('package.json', 'r') as content_file:
            content = content_file.read()
            object_pairs_hook = lambda pairs: OrderedDict( pairs )
            content = json.loads( content, object_pairs_hook=object_pairs_hook )
        for f in ( 'name', 'version' ):
            if f in content and f not in config:
                config[f] = content[f]

    def updateProjectConfig( self, config, updates ) :
        def updater( json ):
            for f in ( 'name', 'version' ):
                if f in updates :
                        json[f] = updates[f]

        return self._updateJSONConfig( 'package.json', updater )
    
    def onPrepare( self, config ):
        npmBin = config['env']['npmBin']
        self._callExternal( [ npmBin, 'install' ] )
        
    def onPackage( self, config ):
        npmBin = config['env']['npmBin']
        self._callExternal( [ npmBin, 'prune', '--production' ] )





from ..buildtool import BuildTool
from .npmtoolmixin import NpmToolMixIn

class gulpTool( NpmToolMixIn, BuildTool ):
    def autoDetect( self, config ) :
        return self._autoDetectByCfg( config, 'gulpfile.js' )
    
    def onBuild( self, config ):
        gruntBin = config['env']['gruntBin']
        self._callExternal( [ gruntBin ] )






from ..buildtool import BuildTool
from .npmtoolmixin import NpmToolMixIn

class gruntTool( NpmToolMixIn, BuildTool ):
    def autoDetect( self, config ) :
        return self._autoDetectByCfg(
                config,
                [ 'Gruntfile.json', 'Gruntfile.coffee' ]
        )
    def onBuild( self, config ):
        gruntBin = config['env']['gruntBin']
        self._callExternal( [ gruntBin ] )
            